"use client";

import { GlassCard } from "@/components/ui/glass-card";
import { Button } from "@/components/ui/button";
import { useState, useRef, useEffect } from "react";
import axios from "axios";

// 在文件顶部导入区域之后，组件定义之前添加
const API_BASE_URL = process.env.NEXT_PUBLIC_API_URL || "http://localhost:8000/api";

// 消息类型定义
interface Message {
  id: string;
  content: string;
  role: "user" | "assistant" | "system";
  timestamp: Date;
}

// 在文件顶部添加这个接口
interface ApiMessage {
  id: number;
  role: string;
  content: string;
  timestamp: string;
}

// 添加一个接口来定义来自API的消息格式
interface ApiResponseMessage {
  role: string;
  content: string;
}

// 系统预设消息 - 定义AI助手的角色和功能
const SYSTEM_PROMPT = `你是"文献启明星"科研助手系统中的AI顾问，专为科研人员、学者和学生设计。
你的主要职责包括：
1. 解答学术和研究方法相关问题
2. 提供文献检索和分析建议
3. 解释复杂的科研概念和理论
4. 帮助分析研究方向和趋势
5. 辅助论文写作和结构规划

你拥有丰富的科研知识，可以帮助用户提高研究效率。在回答问题时，你应保持严谨、专业的态度，同时用友好的语气与用户交流。

文献启明星系统的主要功能：
- 文献研究：智能分析研究领域，生成研究方向报告，查找相关论文
- AI聊天：即你现在提供的服务，解答学术问题，提供研究建议
- 论文分析：自动提取论文关键信息，生成摘要和解读，推荐相关文献

请记住，你是用户科研工作的助手，目标是帮助他们更高效地完成研究工作。`;

// 添加重试逻辑
const fetchWithRetry = async (
  url: string, 
  options: RequestInit, 
  maxRetries = 3
): Promise<Response> => {
  let lastError;
  
  for (let i = 0; i < maxRetries; i++) {
    try {
      return await fetch(url, options);
    } catch (error) {
      console.log(`请求失败，尝试重试 ${i + 1}/${maxRetries}`);
      lastError = error;
      // 指数退避策略
      await new Promise(r => setTimeout(r, 1000 * Math.pow(2, i)));
    }
  }
  
  throw lastError;
};

export default function AIChatPage() {
  // 状态管理
  const [messages, setMessages] = useState<Message[]>([
    {
      id: `system-${Date.now()}-${Math.random().toString(36).substring(2, 9)}`,
      content: SYSTEM_PROMPT,
      role: "system",
      timestamp: new Date(),
    },
    {
      id: `welcome-${Date.now()}-${Math.random().toString(36).substring(2, 9)}`,
      content: "您好！我是文献启明星的AI研究助手。作为您的科研伙伴，我可以帮助您解答学术问题、分析研究方向、提供文献建议，以及辅助论文写作。请问今天有什么我可以协助您的研究工作吗？",
      role: "assistant",
      timestamp: new Date(),
    },
  ]);
  const [inputText, setInputText] = useState("");
  const [isLoading, setIsLoading] = useState(false);
  const [error, setError] = useState<string | null>(null);
  const [visibleMessages, setVisibleMessages] = useState<Message[]>([]);
  
  const messagesEndRef = useRef<HTMLDivElement>(null);
  const chatContainerRef = useRef<HTMLDivElement>(null);
  
  // 在组件的状态变量之后添加一个保存聊天的状态
  const [chatId, setChatId] = useState<number | null>(null);
  const [chatTitle, setChatTitle] = useState("新对话");
  const [isSaving, setIsSaving] = useState(false);
  
  // 过滤系统消息，只显示用户和助手的消息
  useEffect(() => {
    setVisibleMessages(messages.filter(msg => msg.role !== "system"));
  }, [messages]);
  
  // 自动滚动到最新消息
  useEffect(() => {
    if (messagesEndRef.current) {
      messagesEndRef.current.scrollIntoView({ behavior: "smooth" });
    }
  }, [visibleMessages]);
  
  // 添加从URL获取聊天ID的逻辑
  useEffect(() => {
    const urlParams = new URLSearchParams(window.location.search);
    const id = urlParams.get('id');
    if (id) {
      setChatId(parseInt(id));
      // 加载已存在的聊天
      loadExistingChat(parseInt(id));
    }
  }, []);
  
  // 添加加载现有聊天的函数
  const loadExistingChat = async (id: number) => {
    try {
      const token = localStorage.getItem("token");
      if (!token) return;
      
      const response = await axios.get(`${API_BASE_URL}/chat-history/${id}`, {
        headers: {
          Authorization: `Bearer ${token}`
        }
      });
      
      if (response.data) {
        const chatData = response.data;
        setChatTitle(chatData.title);
        
        // 设置消息，保留系统消息
        const systemMessage = messages.find(m => m.role === 'system');
        const loadedMessages = chatData.messages.map((msg: ApiMessage) => ({
          id: `loaded-${msg.id}-${Math.random().toString(36).substring(2, 9)}`,
          content: msg.content,
          role: msg.role,
          timestamp: new Date(msg.timestamp)
        }));
        
        if (systemMessage) {
          setMessages([systemMessage, ...loadedMessages]);
        } else {
          setMessages(loadedMessages);
        }
      }
    } catch (err: any) {
      console.error("加载聊天历史失败:", err);
      setError("无法加载聊天历史，可能已被删除或您无权访问");
    }
  };
  
  // 发送消息
  const sendMessage = async () => {
    if (!inputText.trim()) return;
    
    const uniqueUserId = `user-${Date.now()}-${Math.random().toString(36).substring(2, 9)}`;

    const userMessage = {
      id: uniqueUserId,  // 使用时间戳+随机字符串确保唯一性
      content: inputText.trim(),
      role: "user" as const,
      timestamp: new Date(),
    };
    
    setMessages(prev => {
      // 检查是否已经存在相同ID的消息
      if (prev.some(msg => msg.id === userMessage.id)) {
        return prev;
      }
      return [...prev, userMessage];
    });
    setInputText("");
    setIsLoading(true);
    setError(null);
    
    try {
      // 添加请求详细日志
      console.log("发送到AI服务的消息:", {
        messages: messages.filter(m => m.role !== 'system').concat(userMessage)
          .map(m => ({role: m.role, content: m.content})),
        systemMessage: messages.find(m => m.role === 'system')?.content
      });
      
      const response = await axios.post(`${API_BASE_URL}/chat`, {
        messages: [
          ...messages.filter(m => m.role !== 'system'),
          userMessage
        ].map(m => ({
          role: m.role,
          content: m.content
        })),
        systemMessage: messages.find(m => m.role === 'system')?.content,
      });
      
      console.log("AI服务原始响应:", response.data);
      
      // 增强的响应格式检查和错误处理
      if (!response.data) {
        throw new Error("AI返回了空响应");
      }
      
      // 尝试处理不同的响应格式
      let aiContent = "";
      
      // 检查响应中的messages数组
      if (response.data.messages && Array.isArray(response.data.messages)) {
        // 找到最后一个assistant消息 - 添加明确的类型
        const assistantMessage = response.data.messages
          .filter((msg: ApiResponseMessage) => msg.role === 'assistant')
          .pop();
          
        if (assistantMessage && assistantMessage.content) {
          aiContent = assistantMessage.content;
        }
      } 
      // 如果没有找到messages数组或assistant消息，尝试其他格式
      else if (typeof response.data === 'string') {
        aiContent = response.data;
      } else if (response.data.content) {
        aiContent = response.data.content;
      } else if (response.data.message) {
        aiContent = response.data.message;
      } else if (response.data.text) {
        aiContent = response.data.text;
      } else if (response.data.choices && Array.isArray(response.data.choices) && response.data.choices.length > 0) {
        // OpenAI风格格式
        const choice = response.data.choices[0];
        aiContent = choice.message?.content || choice.text || "";
      }
      
      if (!aiContent) {
        console.error("无法从响应中提取AI回复:", response.data);
        throw new Error("AI返回的响应格式不正确或内容为空");
      }
      
      const uniqueId = `assistant-${Date.now()}-${Math.random().toString(36).substring(2, 9)}`;
      
      const assistantMessage = {
        id: uniqueId,  // 使用时间戳+随机字符串确保唯一性
        content: aiContent,
        role: "assistant" as const,
        timestamp: new Date(),
      };
      
      setMessages(prev => {
        // 检查是否已经存在相同ID的消息
        if (prev.some(msg => msg.id === assistantMessage.id)) {
          return prev;
        }
        return [...prev, assistantMessage];
      });
      
      // 保存聊天历史
      await saveChatToHistory(userMessage, assistantMessage);
      
    } catch (err: any) {
      console.error("AI聊天请求失败:", err);
      if (err.response) {
        console.error("服务器响应:", err.response.status, err.response.data);
      }
      setError(`与AI服务通信失败: ${err.message || "未知错误"}`);
      // 即使API失败，也将用户消息添加到聊天记录
      setMessages(prev => {
        // 检查是否已经存在相同ID的消息
        if (prev.some(msg => msg.id === userMessage.id)) {
          return prev;
        }
        return [...prev, userMessage];
      });
    } finally {
      setIsLoading(false);
    }
  };
  
  // 修改 saveChatToHistory 函数
  const saveChatToHistory = async (userMsg: Message, assistantMsg: Message) => {
    try {
      const token = localStorage.getItem("token");
      if (!token) return; // 未登录不保存
      
      // 确保角色名称符合后端期望的格式
      const formatRole = (role: string): string => {
        // 确保角色只能是这三种值之一
        if (role === "user" || role === "assistant" || role === "system") {
          return role;
        }
        // 默认返回 "user"
        return "user";
      };
      
      // 安全地获取内容，防止undefined错误
      const getSafeContent = (msg: Message | undefined): string => {
        if (!msg || typeof msg.content !== 'string') {
          return "";
        }
        return msg.content;
      };
      
      // 如果是新对话，先创建聊天历史
      if (!chatId) {
        // 使用用户消息作为标题，添加安全检查
        const title = userMsg && userMsg.content ? 
          (userMsg.content.length > 30 ? userMsg.content.substring(0, 30) + "..." : userMsg.content) :
          "新对话";
        
        const simplifiedMessages = [
          {
            role: formatRole(userMsg?.role || "user"),
            content: getSafeContent(userMsg)
          }
        ];
        
        // 只有当assistantMsg存在时才添加到消息列表中
        if (assistantMsg && assistantMsg.content) {
          simplifiedMessages.push({
            role: formatRole(assistantMsg.role || "assistant"),
            content: getSafeContent(assistantMsg)
          });
        }
        
        // 创建新聊天历史
        const response = await axios.post(
          `${API_BASE_URL}/chat-history/`, 
          {
            title: title,
            messages: simplifiedMessages
          },
          {
            headers: { Authorization: `Bearer ${token}` }
          }
        );
        
        if (response.data && response.data.id) {
          setChatId(response.data.id);
          setChatTitle(title);
          
          // 更新URL
          const url = new URL(window.location.href);
          url.searchParams.set('id', response.data.id.toString());
          window.history.pushState({}, '', url);
        }
      } else {
        // 已有聊天历史，添加新消息
        if (userMsg && userMsg.content) {
          try {
            // 添加用户消息
            await axios.post(
              `${API_BASE_URL}/chat-history/${chatId}/messages`, 
              {
                role: formatRole(userMsg.role),
                content: getSafeContent(userMsg)
              },
              {
                headers: { Authorization: `Bearer ${token}` }
              }
            );
          } catch (msgErr: any) {
            console.error("添加用户消息失败:", msgErr);
            if (msgErr.response) {
              console.error("错误详情:", msgErr.response.status, msgErr.response.data);
            }
          }
        }
        
        if (assistantMsg && assistantMsg.content) {
          try {
            // 添加AI助手消息
            await axios.post(
              `${API_BASE_URL}/chat-history/${chatId}/messages`, 
              {
                role: formatRole(assistantMsg.role),
                content: getSafeContent(assistantMsg)
              },
              {
                headers: { Authorization: `Bearer ${token}` }
              }
            );
          } catch (msgErr: any) {
            console.error("添加AI助手消息失败:", msgErr);
            if (msgErr.response) {
              console.error("错误详情:", msgErr.response.status, msgErr.response.data);
            }
          }
        }
      }
    } catch (err: any) {
      console.error("保存聊天历史失败:", err);
      if (err.response) {
        console.error("错误详情:", err.response.data);
      }
    }
  };
  
  // 处理按Enter键发送
  const handleKeyPress = (e: React.KeyboardEvent) => {
    if (e.key === "Enter" && !e.shiftKey) {
      e.preventDefault();
      sendMessage();
    }
  };
  
  // 清空聊天记录
  const clearChat = () => {
    setMessages([
      {
        id: `system-${Date.now()}-${Math.random().toString(36).substring(2, 9)}`,
        content: SYSTEM_PROMPT,
        role: "system",
        timestamp: new Date(),
      },
      {
        id: `welcome-${Date.now()}-${Math.random().toString(36).substring(2, 9)}`,
        content: "聊天已重置。我是文献启明星的AI研究助手，请问有什么我可以帮助您的？",
        role: "assistant",
        timestamp: new Date(),
      }
    ]);
    
    // 重置聊天ID和标题
    setChatId(null);
    setChatTitle("新对话");
    
    // 移除URL中的id参数
    const url = new URL(window.location.href);
    url.searchParams.delete('id');
    window.history.pushState({}, '', url);
  };
  
  return (
    <main className="relative flex min-h-[calc(100vh-64px)] flex-col">
      <div className="container mx-auto px-4 pb-4 flex flex-col h-[calc(100vh-64px)]">
        {/* 标题区域 - 减小高度并固定在顶部 */}
        <div className="text-center py-4 flex justify-between items-center">
          <div className="flex-1"></div>
          <div className="flex-2 text-center">
            <h1 className="text-3xl font-bold text-center text-transparent bg-clip-text bg-gradient-to-r from-white to-white/70 drop-shadow-lg">
              AI 智能助手
            </h1>
            <p className="text-sm text-center text-white/80">
              您的个人研究与学习伙伴
            </p>
            <div className="w-12 h-1 mx-auto mt-2 bg-gradient-to-r from-[var(--theme-accent)] to-[var(--theme-highlight)] rounded-full"></div>
          </div>
          <div className="flex-1 flex justify-end">
            <button
              onClick={clearChat}
              className="text-white/70 hover:text-white text-sm py-1 px-3 rounded-md bg-white/10 hover:bg-white/20 transition-colors"
            >
              <svg xmlns="http://www.w3.org/2000/svg" width="16" height="16" viewBox="0 0 24 24" fill="none" stroke="currentColor" strokeWidth="2" strokeLinecap="round" strokeLinejoin="round" className="inline-block mr-1">
                <path d="M3 6h18"></path>
                <path d="M19 6v14c0 1-1 2-2 2H7c-1 0-2-1-2-2V6"></path>
                <path d="M8 6V4c0-1 1-2 2-2h4c1 0 2 1 2 2v2"></path>
              </svg>
              清空
            </button>
          </div>
        </div>
        
        {/* 聊天区域 - 占据大部分空间 */}
        <div className="flex-1 overflow-hidden flex flex-col">
          <GlassCard className="w-full flex-1 flex flex-col overflow-hidden">
            {/* 聊天消息区域 - 可滚动区域 */}
            <div 
              ref={chatContainerRef}
              className="flex-1 overflow-y-auto px-2 py-4 scroll-smooth" 
              style={{ 
                scrollbarWidth: 'thin',
                scrollbarColor: 'rgba(255,255,255,0.2) transparent'
              }}
            >
              <div className="space-y-6">
                {visibleMessages.map((message) => (
                  <div 
                    key={message.id} 
                    className={`flex items-start gap-3 ${message.role === "user" ? "justify-end" : ""}`}
                  >
                    {message.role === "assistant" && (
                      <div className="w-8 h-8 rounded-full bg-[var(--theme-accent)]/80 flex items-center justify-center text-white text-sm font-medium shrink-0 mt-1">AI</div>
                    )}
                    
                    <div className={`${
                      message.role === "assistant" 
                        ? "bg-white/10 backdrop-blur-sm rounded-lg rounded-tl-none" 
                        : "bg-[var(--theme-accent)]/20 backdrop-blur-sm rounded-lg rounded-tr-none"
                    } p-3 max-w-[85%]`}>
                      <p className="text-white/90 whitespace-pre-wrap break-words">{message.content}</p>
                    </div>
                    
                    {message.role === "user" && (
                      <div className="w-8 h-8 rounded-full bg-white/20 flex items-center justify-center text-white text-sm font-medium shrink-0 mt-1">您</div>
                    )}
                  </div>
                ))}
                
                {isLoading && (
                  <div className="flex items-start gap-3">
                    <div className="w-8 h-8 rounded-full bg-[var(--theme-accent)]/80 flex items-center justify-center text-white text-sm font-medium shrink-0">AI</div>
                    <div className="bg-white/10 backdrop-blur-sm p-3 rounded-lg rounded-tl-none">
                      <div className="flex space-x-2 items-center h-6">
                        <div className="w-2 h-2 rounded-full bg-white/70 animate-bounce"></div>
                        <div className="w-2 h-2 rounded-full bg-white/70 animate-bounce" style={{animationDelay: "0.2s"}}></div>
                        <div className="w-2 h-2 rounded-full bg-white/70 animate-bounce" style={{animationDelay: "0.4s"}}></div>
                      </div>
                    </div>
                  </div>
                )}
                
                {error && (
                  <div className="bg-red-500/20 p-3 rounded-lg text-white/90 text-sm mx-4">
                    <span className="font-medium">错误：</span> {error}
                  </div>
                )}
                
                {/* 用于自动滚动的空白元素 */}
                <div ref={messagesEndRef} />
              </div>
            </div>
            
            {/* 输入区域 - 固定在底部 */}
            <div className="p-4 border-t border-white/10">
              <div className="flex gap-2">
                <textarea 
                  className="w-full bg-white/10 text-white border border-white/20 rounded-lg p-3 focus:outline-none focus:ring-2 focus:ring-[var(--theme-accent)]/50 resize-none h-16"
                  placeholder="输入您的科研问题或需求..."
                  value={inputText}
                  onChange={(e) => setInputText(e.target.value)}
                  onKeyDown={handleKeyPress}
                  disabled={isLoading}
                ></textarea>
                <Button 
                  className="self-end"
                  onClick={sendMessage}
                  disabled={isLoading || !inputText.trim()}
                >
                  <svg xmlns="http://www.w3.org/2000/svg" width="20" height="20" viewBox="0 0 24 24" fill="none" stroke="currentColor" strokeWidth="2" strokeLinecap="round" strokeLinejoin="round">
                    <path d="M22 2L11 13"></path>
                    <path d="M22 2L15 22L11 13L2 9L22 2Z"></path>
                  </svg>
                </Button>
              </div>
              <div className="mt-2 text-white/50 text-xs">
                提示：您可以询问学术问题、研究方向分析、文献推荐、论文写作建议等。按Enter发送，Shift+Enter换行。
              </div>
            </div>
          </GlassCard>
        </div>
        
        {/* 底部版权信息 */}
        <p className="text-white/50 text-center text-xs mt-2">
          基于DeepSeek AI的智能对话系统 | 文献启明星科研助手
        </p>
      </div>
    </main>
  );
} 